%clc
%clear all
%close all


%axes(handles.axes37)


% Constant Fluid Properties - Drilling Fluid
qO_SC = 250/42*60*24;                               % Oil Circulating Rate, gpm to stb/D
qG_SC = 600*60*24;                                  % Gas Circulating Rate, scfpm to scf/D

Rsi = qG_SC/qO_SC; 

GASTYPE = 'CH4';                                    % Gas in system
yG = 0.66;                                          % Specific Gravity Gas
Mg = 16.04;                                         % Molar Mass Gas
API = 30;                                           % API Gravity Oil
yO = 141.5/(API+131.5);                             % Specific Gravity Oil

%% Data
limit = 0.01;                                       % Iteration limit, psi
timemax = 500;                                      % Number of Timesteps

% Injection Rates
qO_SC_NEW = [50*ones(1,500)]*60*24/42;             % Oil Injection Rate, gpm to stb/D
qG_SC_NEW = [600*ones(1,15) 20*600*ones(1,5)...     % Gas Injection Rate, scfpm to scf/D
    600*ones(1,480)]*24*60;   
%qG_SC_NEW = [];

% Choke data
d = 64.59;                                          % Initial Choke Opening, 1/64 inch
P_Choke = choke(qO_SC, qG_SC/qO_SC, d);             % Initial Back Pressure, psi
dCH = [64.59*ones(1,500)];                          % Choke Openings, 1/64 inch

% Reservoir Data
Qmax = 4000;                                        % Max Flow Rate, Reservoir, stb/D
Presavg = 3400;                                     % Reservoir Pressure, psi
GOR = 5000;                                         % Gas-Oil Ratio scf/bbl

inflow_o = zeros(1,timemax);                        % Inflow Vector Liquid, bbl/d
inflow_g = zeros(1,timemax);                        % Inflow Vector Gas, scf/d
BHP = zeros(1,timemax);                             % BHP Vector, psi

t = 0;                                              % Time Counter Initiated
ds0 = newgrid(77,1.5,t);
%ds0 = newgrid2(t);                  % Get Grid for Calculations
intervals = size(ds0,1);                            % Number of Depth Intervals

qG = zeros(1,intervals);                            % Gas Flow Rate, ft3/sec
qO = zeros(1,intervals);                            % Liquid Flow Rate, ft3/sec
qT = zeros(1,intervals);                            % Total Flow Rate, ft3/sec
rhoL = zeros(1,intervals);                          % Liquid Density, lbm/cuft
rhoG = zeros(1,intervals);                          % Gas Density, lbm/cuft
rhoM = zeros(1,intervals);                          % Mixture Density, lbm/cuft
Rs = zeros(1,intervals);                            % GOR, scf/STB
Bo = zeros(1,intervals);                            % FVF, bbl/STB
z = zeros(1,intervals);                             % Compressibility Factor
vsL = zeros(1,intervals);                           % Liquid Superficial Velocity, ft/sec
vsG = zeros(1,intervals);                           % Gas Superficial Velocity, ft/sec
NLV = zeros(1,intervals);                           % Flow Pattern Parameter



%% Simulation Flow Up Annulus (Simulated downwards)

for ii = 1:intervals                                                        % Loop over all intervals

    deviation = 1000;                                                       % Pressure deviation, psi
    ds0(ii,20) = 0;                                                         % Iteration Counter Initiated
    
    while(abs(deviation)>limit)                                             % Checks if iteration limit is reached
        
        ds0(ii,20) = ds0(ii,20) + 1;                                        % Iteration Counter Increases
        
        % Section Estimates the Average Pressure
        if (ii>1)
            ds0(ii,15) = ds0(ii-1,16);                                      % Inlet Pressure Equals Outles Pressure Last Interval
        else
            ds0(ii,15) = P_Choke;                                           % Inlet Pressure Equals Choke Pressure at Surface
        end
        ds0(ii,16) = ds0(ii,15) + ds0(ii,14);                               % Calculates Outlet Pressure from Inlet and Guessed Change, psi
        ds0(ii,17) = (ds0(ii,15) + ds0(ii,16))/2;                           % Average Pressure, psi
       
        % Section Calculate Fluid Properties @ Pavg, Tavg
        pb = p_bubble(Rsi,yG,ds0(ii,9),API);
        [Rs(ii), Bo(ii)] = standing(yG, API, pb, ds0(ii,17), ds0(ii,9));         % GOR, scf/STB & FVF, bbl/STB
        z(ii) = zfac4(GASTYPE,ds0(ii,9),ds0(ii,17));                        % Gas Compressibility
        [qG(ii), qO(ii), qT(ii), rhoL(ii), rhoG(ii), rhoM(ii)] = ...        % Gas/Liquid/Total Flow Rates, cuft/sec
             fluid_properties(yG, yO, qO_SC, qG_SC,...                      % Gas/Liquid/Total Densities, lbm/cuft
             ds0(ii,17), ds0(ii,9), Rs(ii), Bo(ii), z(ii),Rsi);                                    
        %pb = p_bubble(Rs(ii),yG,ds0(ii,9),API);                             % Bubble Point Pressure, psi
        visO = real(oilviscosity(API,ds0(ii,9),Rs(ii),ds0(ii,17),pb));      % Oil Viscosity, cp
        visG = gasviscosity(rhoG(ii), Mg, ds0(ii,9));                       % Gas Viscosity, cp
        sigma = IFT(API,ds0(ii,17),ds0(ii,9));                              % Interfacial Tension, dynes/cm^2
        
        % Section Calculates Flow Parameters for Beggs-Brill Corrolation
        vsL(ii) = qO(ii)./ds0(ii,12);                                       % Superficial Velocity Liquid, ft/sec
        vsG(ii) = qG(ii)./ds0(ii,12);                                       % Superficial Velocity Gas, ft/sec     
        NLV(ii) = 1.938.*vsL(ii).*(rhoL(ii)./sigma)^(0.25);                 % Flow Pattern Parameter
        
        % Section Calculates the Pressure Drop From Beggs-Brill Corrolation
        ds0(ii,18) = beggs_brill(vsG(ii), vsL(ii), rhoG(ii), rhoL(ii),...
                 visO, visG, NLV(ii), ds0(ii,17), ds0(ii,13),...
                 (ds0(ii,10)-ds0(ii,11)));                                  % Pressure Gradient, psi/ft
        ds0(ii,19) = ds0(ii,18).*ds0(ii,4);                                 % Calculated Pressure Change Over Interval, psi
        deviation = ds0(ii,14)-ds0(ii,19);                                  % Pressure Deviation From Guess, psi 
        ds0(ii,14) = ds0(ii,19);                                            % New Guess Pressure Change (next iteration), psi

        % Section Tries Average Value of Two Iterations if Convergence Fails
        if (ds0(ii,20) == 201)
            dev1 = deviation;                                               % Holds Calculated Deviation
            val1 = ds0(ii,14);                                              % Holds Calculated Pressure
        elseif (ds0(ii,20) == 202)
            dev2 = deviation;                                               % Holds Calculated Deviation, psi
            val2 = ds0(ii,14);                                              % Holds Calculated Pressure, psi
            ds0(ii,14) = mean([val1 val2]);                                 % Average Pressure of Two Last Iterations, psi
            deviation = mean([dev1 dev2]);                                  % Average Deviation of Two Last Iterations, psi
        end
    end
    
    ds0(ii,16) = ds0(ii,15) + ds0(ii,19);                                   % Outlet Pressure of Interval, psi
    
end

eval(['data_time_' num2str(t) '=ds0;']);                                    % Saves Calculated Data for Current Timestep
BHP0 = ds0(intervals,16);                                                   % Estimated BHP (Outlet Pressure Last Interval), psia
[inflow_oint, inflow_gint] = inflow(Qmax, BHP0, Presavg, GOR);              % Calculates Reservoir Fluid Inflow (Next Timestep)

qG = zeros(intervals,timemax);                                              % Gas Flow Rate, ft3/sec
qO = zeros(intervals,timemax);                                              % Liquid Flow Rate, ft3/sec
qT = zeros(intervals,timemax);                                              % Total Flow Rate, ft3/sec
rhoL = zeros(intervals,timemax);                                            % Liquid Density, lbm/cuft
rhoG = zeros(intervals,timemax);                                            % Gas Density, lbm/cuft
rhoM = zeros(intervals,timemax);                                            % Mixture Density, lbm/cuft
Rs = zeros(intervals,1);                                                    % GOR, scf/STB
Bo = zeros(intervals,1);                                                    % FVF, bbl/STB
z = zeros(intervals,1);                                                     % Compressibility Factor
vsL = zeros(intervals,timemax);                                             % Liquid Superficial Velocity, ft/sec
vsG = zeros(intervals,timemax);                                             % Gas Superficial Velocity, ft/sec
NLV = zeros(intervals,timemax);                                             % Flow Pattern Parameter

A = zeros(intervals,t);                                                     % Matrix To Hold Inflow Fluid Properties 

%% Simulation Flow Up Annulus (Simulated Upwards - Negative Direction)

for t = 1:timemax                                                           % Loop Over All Timesteps

    dst = newgrid(77,1.5,t);
    %dst = newgrid2(t);                                      % Get Grid for Calculations
    intervals = size(dst,1);                                                % Number of Depth Intervals
  
    
    GN_GUI = 600; %get(handles.slider1, 'Value');
    qG_SC_NEW(t) = GN_GUI*24*60;

    if t == 1
        BHP_guess = BHP0;                                                   % BHP Assumed Equal to Previous Timestep, psi
        qOtot_btm(t) = qO_SC_NEW(t) + inflow_oint;                          % Liquid Circulation Rate, Inflow + Injection, stb/D
        qGtot_btm(t) = qG_SC_NEW(t) + inflow_gint;                          % Gas Circulation Rate, Inflow + Injection, scf/D
    else
        BHP_guess = BHP(t-1);                                               % BHP Assumed Equal to Previous Timestep, psi
        qOtot_btm(t) = qO_SC_NEW(t) + inflow_o(t-1);                        % Liquid Circulation Rate, Inflow + Injection, stb/D
        qGtot_btm(t) = qG_SC_NEW(t) + inflow_g(t-1);                        % Gas Circulation Rate, Inflow + Injection, scf/D
    end
    
    devC = 100;                                                             % Choke Devaiation, psi
    while (abs(devC)>1)                                                     % Checks if Iteration Limit is Reached
        time = t;                                                           % Holds Timestep in Seperate Variable

        for ii = intervals:-1:1                                             % Loop over all intervals

            deviation = 1000;                                               % Pressure deviation, psia
            dst(ii,20) = 0;                                                 % Iteration Counter Initiated

            while(abs(deviation)>limit)                                     % Checks if Iteration Limit is Reached

                dst(ii,20) = dst(ii,20) + 1;                                % Iteration Counter Increases
                
                % Section Estimates the Average Pressure
                if (ii ~= intervals)
                    dst(ii,15) = dst(ii+1,16);                              % Inlet Pressure Equals Outles Pressure Last Interval
                else
                    dst(ii,15) = BHP_guess;                                 % Inlet Pressure Equals Guessed BHP
                end
                dst(ii,16) = dst(ii,15) + dst(ii,14);                       % Calculates Outlet Pressure from Inlet and Guessed Change, psi
                dst(ii,17) = (dst(ii,15) + dst(ii,16))/2;                   % Average Pressure, psia

                % Section Finds the Fluid Properties (Pavg, Tavg) for Current Inflow at all Intervals
                pb = p_bubble(Rsi,yG,dst(ii,9),API);
                [Rs(ii), Bo(ii)] = standing(yG, API, pb, dst(ii,17), dst(ii,9)); % GOR, scf/STB & FVF, bbl/STB
                z(ii) = zfac4(GASTYPE,dst(ii,9),dst(ii,17));                % Gas Compressibility
                [qGn, qOn, qTn, rhoOn, rhoGn, rhoMn] = ...                  % Gas/Liquid/Total Flow Rates, cuft/sec            % Calculate Fluid Properties @ Pavg, Tavg
                   fluid_properties(yG, yO, qOtot_btm(t), qGtot_btm(t),...  % Gas/Liquid/Total Densities, lbm/cuft             % Flow Rates Out, Gas/Liquid/Total, cuft/sec (Input as given)
                   dst(ii,17), dst(ii,9), Rs(ii), Bo(ii), z(ii),Rsi);              

                %  Section Saves Values for Future Timesteps 
                qGss(ii,t) = qGn;                                           % Gas Flow Rate, cuft/sec
                qOss(ii,t) = qOn;                                           % LiquidFlow Rate, cuft/sec
                qTss(ii,t) = qTn;                                           % Total Flow Rate, cuft/sec
                rhoOss(ii,t) = rhoOn;                                       % Liquid Density, lbm/cuft
                rhoGss(ii,t) = rhoGn;                                       % Gas Density, lbm/cuft
                rhoMss(ii,t) = rhoMn;                                       % Mixture Density, lbm/cuft

                % Section Find Which Timestep,t, the Flow in Interval, i, Corresponds to
                % ASSUMPTION, Only lower interval produces formation fluid
                % time - Correct Inflow  
                dt = 60;                                                    % Length of Timestep, sec
                M_influx(ii,t) = qTss(intervals,t)*dt.*rhoMss(intervals,t); % Mass Influx of Current Timestep, lbm/min
                Vol(ii) = dst(ii,12).*dst(ii,4);                            % Volume Capacity of Current Interval, ft3
                Mass_cap(ii,t) = Vol(ii)*rhoMss(ii,t);                      % Mass Capacity of Current Influx/Interval, lbm
                ShiftRate(ii,t) = M_influx(ii,t)./Mass_cap(ii,t);           % Number of Intervals Influx Fills
                k = find(isnan(ShiftRate));                                 % Replaces Dummy Data 
                ShiftRate(k) = 0;

                time = t;                                                   % Assumes Current Influx Fills Current Interval

                if time>1                                                   % At First Timestep No Flow Has Shifted
                    A(:,time) = A(:,time-1);                                % Collects Flow Posistion From Last Timestep
                    A(:,time) = shift(A(:,time),round(ShiftRate(time)));    % Shifts Flow Posistion Upwards in Annulus
                end

                FirstZeroElement = intervals -round(ShiftRate(time)-1);     % Finds First Interval Not Filled With Current Influx  
                if (FirstZeroElement<1)                                     % If Entire Well is Filled With Current Inflow...
                    FirstZeroElement = 1;                                   % .. Assign First Interval (top) to Be Top of Influx
                end
                A(FirstZeroElement:intervals,time) = time;                  % Assignes Current Inflow To All Affected (Lower) Intervals
                time = A(ii,t);                                             % Assigns Correct Influx to Correct Interval and Time

                if time>0
                    qG(ii,t) = qGss(ii,time);                               % Gas Flow Rate, cuft/sec
                    qO(ii,t) = qOss(ii,time);                               % Liquid Flow Rate, cuft/sec
                    qT(ii,t) = qTss(ii,time);                               % Total Flow Rate, cuft/sec
                    rhoL(ii,t) = rhoOss(ii,time);                           % Liquid Density, lbm/cuft
                    rhoG(ii,t) = rhoGss(ii,time);                           % Gas Density, lbm/cuft
                    rhoM(ii,t) = rhoMss(ii,time);                           % Mixture Density, lbm/cuft
                else
                    qG(ii,t) = qGss(ii,1);                                  % Gas Flow Rate, cuft/sec
                    qO(ii,t) = qOss(ii,1);                                  % Liquid Flow Rate, cuft/sec
                    qT(ii,t) = qTss(ii,1);                                  % Total Flow Rate, cuft/sec
                    rhoL(ii,t) = rhoOss(ii,1);                              % Liquid Density, lbm/cuft
                    rhoG(ii,t) = rhoGss(ii,1);                              % Gas Density, lbm/cuft
                    rhoM(ii,t) = rhoMss(ii,1);                              % Mixture Density, lbm/cuft
                end

                % Section Calculates More Fluid Properties @ Pavg, Tavg
                pb = p_bubble(Rs(ii),yG,dst(ii,9),API);                     % Bubble Point Pressure, psi
                visO = oilviscosity(API,dst(ii,9),Rs(ii),dst(ii,17),pb);    % Oil Viscosity, cp
                visG = gasviscosity(rhoG(ii,t), Mg, dst(ii,9));             % Gas Viscosity, cp
                sigma = IFT(API,dst(ii,17),dst(ii,9));                      % Interfacial Tension, dynes/cm^2
        
                % Section Calculates Flow Parameters for Beggs-Brill Corrolation
                vsL(ii,t) = qO(ii,t)./dst(ii,12);                           % Superficial Velocity Liquid, ft/sec
                vsG(ii,t) = qG(ii,t)./dst(ii,12);                           % Superficial Velocity Gas, ft/sec
                NLV(ii,t) = 1.938*vsL(ii,t).*(rhoL(ii,t)/sigma)^(0.25);     % Flow Pattern Parameter

                % Section Calculates the Pressure Drop From Beggs-Brill Corrolation
                dst(ii,18) = beggs_brill(vsG(ii,t), vsL(ii,t), ...
                    rhoG(ii,t), rhoL(ii,t),visO, visG, NLV(ii,t), ...
                    dst(ii,17), abs(dst(ii,13)),(dst(ii,10)-dst(ii,11)));   % Pressure Gradient, psi/ft
                dst(ii,19) = -dst(ii,18).*dst(ii,4);                        % Calculated Pressure Change Over Interval, psi
                deviation = dst(ii,14)-dst(ii,19);                          % Pressure Deviation From Guess, psi 
                dst(ii,14) = dst(ii,19);                                    % New Guess Pressure Change (next iteration), psi

                % Section Tries Average Value of Two Iterations if Convergence Fails
                if (dst(ii,20) == 201)
                    dev1 = deviation;                                       % Holds Calculated Deviation, psi
                    val1 = dst(ii,14);                                      % Holds Calculated Pressure, psi
                elseif (dst(ii,20) == 202)
                    dev2 = deviation;                                       % Holds Calculated Deviation, psi
                    val2 = dst(ii,14);                                      % Holds Calculated Pressure, psi
                    dst(ii,14) = mean([val1 val2]);                         % Average Pressure of Two Last Iterations, psi
                    deviation = mean([dev1 dev2]);                          % Average Deviation of Two Last Iterations, psi
                end
            end
            dst(ii,16) = dst(ii,15) + dst(ii,19);                           % Outlet Pressure of Interval, psi
        end

        P_Surface = dst(1,16);                                              % Calculated Surface Pressure, psi

        % Section Finds Flow and Actual Pressure Drop Over Choke
        if time == 0
            qL(t) = qO_SC;                                                  % Liquid Flow Rate Over Choke, stb/day
            GLR(t) = qG_SC/qO_SC;                                           % Gas-Liquid Ratio Over Choke, scf/stb
        else
            qL(t) = qOtot_btm(time);                                        % Liquid Flow Rate Over Choke, stb/day
            GLR(t) = qGtot_btm(time)/qOtot_btm(time);                       % Gas-Liquid Ratio Over Choke, scf/stb
        end
        NEWCHOKE(t) = choke(qL(t), GLR(t), dCH(t));                         % Calculated Choke Pressure Drop / Surface Pressure, psi
        devC = NEWCHOKE(t) - P_Surface;                                     % Deviation Choke Pressure, psi
        BHP_guess = BHP_guess + 0.8*devC;                                   % New BHP Guess, Based on Choke Deviation, psi
    end


        eval(['data_time_' num2str(t) '=dst;']);                            % Saves Calculated Data for Current Timestep
        BHP(t) = BHP_guess;                                                 % Estimated BHP (Outlet Pressure Last Interval), psia
        [inflow_o(t), inflow_g(t)] = inflow(Qmax, BHP(t), Presavg, GOR);    % Calculates Reservoir Fluid Inflow (Next Timestep)
        
        fprintf('Timestep t = %d Converged at BHP = %.1f psi\n',t,BHP(t));  % Printout to Screen
%end


%% Plotting

tvector = 0:t;
%tvector = 0:timemax;
Pchoke = [P_Choke NEWCHOKE];
BHPplot = [BHP0 BHP(1:t)];

%inflow_o = [inflow_oint inflow_o];
inflow_o_plot = [inflow_oint inflow_o(1:t)];
%inflow_g = [inflow_gint inflow_g];
inflow_g_plot = [inflow_gint inflow_g(1:t)];
BHPplot = [BHP0 BHP(1:t)];

qOtot_btm_plot = [qO_SC qOtot_btm(1:t)];
qGtot_btm_plot = [qG_SC qGtot_btm(1:t)];
%%

y2 = real(max([max(BHPplot) Presavg]));
y2 = y2 + 200 - mod(y2,200);
y1 = real(min([min(BHPplot) Presavg]));
y1 = y1 - 200 + mod(y1,200);

subplot(4,1,1); 
AX = plotyy([0 t], [Presavg Presavg], tvector, BHPplot, 'plot');
set(get(AX(1),'Ylabel'),'String','Reservoir Pressure, psi')
set(get(AX(2),'Ylabel'),'String','BHP, psi')
set(AX(:), 'ylim', [2500 4500], 'YTickMode','auto')
xlabel('Time, minutes')

subplot(4,1,2); 
AX = plotyy(tvector, Pchoke, tvector, inflow_o_plot, 'plot');
set(get(AX(1),'Ylabel'),'String','Choke Pressure, psi')
set(get(AX(2),'Ylabel'),'String','Oil Produced, bbl/day')
set(AX(1), 'ylim', [-900 2500], 'YTickMode','auto')
set(AX(2), 'ylim', [0 3000], 'YTickMode','auto')
xlabel('Time, minutes')

subplot(4,1,3); 
AX = plotyy(tvector, qGtot_btm_plot, tvector, qOtot_btm_plot, 'plot');
set(get(AX(1),'Ylabel'),'String','Gas Flow, scf/day')
set(get(AX(2),'Ylabel'),'String','Oil Flow, bbl/day')
%set(AX(1), 'ylim', [350 550], 'YTickMode','auto')
set(AX(2), 'ylim', [8500 9500], 'YTickMode','auto')
xlabel('Time, minutes')

subplot(4,1,4)
AX = plotyy(tvector,[d dCH(1:t)], tvector,[d dCH(1:t)]./(4*64)*100, 'plot');
set(get(AX(1),'Ylabel'),'String','Choke Opening, 1/64 inch')
set(get(AX(2),'Ylabel'),'String','Choke Opening, % of 4"')
set(AX(1), 'ylim', [50 150], 'YTickMode','auto')
set(AX(2), 'ylim', [0 100], 'YTickMode','auto')
xlabel('Time, minutes')

pause(0.5)
end