% Script to calculate flow regime of a two-phase fluid
% using Beggs-Brill corrolation for flow in pipes

% Input density, mass flow rate and pipe diameter
% vsG -  Gas Superficial Velocity, ft/sec
% vsL -  Liquid Superficial Velocity, ft/sec
% rhoG - Mass Density, Gas, lbm/cuft
% rhoL - Mass Density, Liquid, lbm/cuft
% visO - Viscosity Oil, cp
% cisG - Viscosity Gas, cp
% NLV - Flow number
% p - pressure, psia
% phi - inclination, degrees
% d - Pipe ID (adjusted for annular flow), ft

% Output - pressure gradient over interval
% dPdZ - Pressure drop, psi/ft

function flow = beggs_brill(vsG, vsL, rhoG, rhoL, visL, visG, NLV, p, phi, d)

g = 32.2;                                       % Gravitational Constant, lbm-ft/lbf-s^2

vm = vsG + vsL;                                 % Mixture Velocity, ft/sec
NFR = vm^2/(g*d);                               % Froude Number 
lambdaL = vsL/vm;                               % Liquid Velocity Fraction

L1 = 316*lambdaL^0.302;                         % Flow Indicator
L2 = 0.0009252*lambdaL^(-2.4684);                % Flow Indicator
L3 = 0.1*lambdaL^(-1.4516);                     % Flow Indicator
L4 = 0.5*lambdaL^(-6.738);                      % Flow Indicator

% Section defines the flow type after flow numbers and indicators 

if((lambdaL < 0.01 && NFR < L1) || (lambdaL >= 0.01 && NFR < L2))
    Flowtype = 'segregated';
elseif((0.01<=lambdaL && lambdaL<0.4 && L3 < NFR && NFR <= L1)...
        || (lambdaL >= 0.4 && L3 < NFR && NFR <= L4))
    Flowtype = 'intermittent';
elseif((lambdaL < 0.4 && NFR >= L1) || (lambdaL >= 0.4 && NFR > L4))
    Flowtype = 'distributed';
elseif(L2 < NFR && NFR < L3)
    Flowtype = 'transition';
else
    Flowtype = 'unknown';                       % Flow Type Not Found
end


% Section defines the flow direction from angle (horizontal = 0)
% Horizontal flow can be calculted as either upwards or downwards

if (phi>0)
    Flowtype = [Flowtype ' uphill'];            % Flow Upwards
else
    Flowtype = [Flowtype ' downhill'];          % Flow Downwards (or horizontal)
end


% Section picks the correct coefficents for each flowtype
% and calculates the liquid holdup

% Flowcoefficients, a, b, c, dm, e, f, gm
% HL - Liquid holdup

switch Flowtype
    
    case {'segregated uphill'}
        a = 0.98;
        b = 0.4846;
        c = 0.0868;
        dm = 0.011;
        e = -3.768;
        f = 3.539;
        gm = -1.614;  
        HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
    
    case {'intermittent uphill'}
        a = 0.845;
        b = 0.5351;
        c = 0.0173;
        dm = 2.96;
        e = 0.305;
        f = -0.4437;
        gm = 0.0978;
        HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
    
    case {'distributed uphill'}
        a = 1.065;
        b = 0.5824;
        c = 0.0609;
        HL = a*lambdaL^b/(NFR^c);  
        if HL < lambdaL
            HL = lambdaL;
        end
        
    case {'transition uphill'}
        A = (L3-NFR)/(L3-L2);
        B = 1-A;
        a = 0.98;
        b = 0.4846;
        c = 0.0868;
        dm = 0.011;
        e = -3.768;
        f = 3.539;
        gm = -1.614;
        HLs = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
        a = 0.845;
        b = 0.5351;
        c = 0.0173;
        dm = 2.96;
        e = 0.305;
        f = -0.4437;
        gm = 0.0978;
        HLi = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
        HL = A*HLs + B*HLi;
    
    case {'segregated downhill'}
        a = 0.98;
        b = 0.4846;
        c = 0.0868;
        dm = 4.70;
        e = -0.3692;
        f = 0.1244;
        gm = -0.5056;
        HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
    
    case {'intermittent downhill'}
        a = 0.845;
        b = 0.5351;
        c = 0.0173;
        dm = 4.70;
        e = -0.3692;
        f = 0.1244;
        gm = -0.5056;
        HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
    
    case{'distributed downhill'}
        a = 1.065;
        b = 0.5824;
        c = 0.0609;
        dm = 4.70;
        e = -0.3692;
        f = 0.1244;
        gm = -0.5056;
        HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
    
    case {'transition downhill'}
        A = (L3-NFR)/(L3-L2);
        B = 1-A;
        a = 0.98;
        b = 0.4846;
        c = 0.0868;
        dm = 4.70;
        e = -0.3692;
        f = 0.1244;
        gm = -0.5056;
        HLs = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
        a = 0.845;
        b = 0.5351;
        c = 0.0173;
        dm = 4.70;
        e = -0.3692;
        f = 0.1244;
        gm = -0.5056;
        HLi = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi);
        HL = A*HLs + B*HLi;   

    otherwise
        error('Unknown Flowtype')               % Terminates the process upon initiation

end
 
% Section corrects liquid holdup for extreme values, value should be between 0 and 1

if HL>1         
    HL = 1;     
elseif HL<0
    HL = 0;
end

% Section calcultes densities and friction factor

rhos = rhoL*HL+rhoG*(1-HL);                             % Density, slip, lbm/cuft
rhon = rhoL*lambdaL + rhoG*(1-lambdaL);                 % Density, no-slip, lbm/cuft
NRen = 1488*rhon*vm*d/(visL*lambdaL+visG*(1-lambdaL));  % Reynolds number, no-slip
fn = 1/(2*log10(NRen/(4.5223*log10(NRen)-3.8215)))^2;   % Friction factor, no-slip
y = lambdaL/HL^2;                                       % Adjustment parameter
y1 = log(y);                                            % Adjustment parameter

if ((1<y) && (y<1.2))
    S = log(2.2*y-1.2);                                 % Slip correction parameter
else
    S = y1/(-0.0523+3.182*y1-0.8725*y1^2+0.01853*y1^4); % Slip correction parameter
end

es = exp(S);                                            % Slip correction
ftp = es*fn;                                            % Friction factor, two-phase flow
Ek = rhos*vm*vsG/(g*p*144);                             % Accelleration pressure drop(factor)

dPdZ_el = rhos*sind(phi);                               % Hydrostatic pressure drop, psi
dPdZ_f = ftp*rhon*vm^2/(2*g*d);                         % Frictional pressure drop, psi

dPdZ = (dPdZ_el + dPdZ_f)/(1-Ek)/144;                   % Total pressure drop, psi

%Flowtype
flow = dPdZ;                                            % Returns Output
end


% Script inside beggs_brill.m to calculate liquid holudup from parameters

% Input flow parameters

% Output liquid holdup
% HL - Liquid holdup , []

function HL = HL_Temp(a,b,c,dm,e,f,gm,NFR,lambdaL, NLV, phi)

    HL_0 = a*lambdaL^b/(NFR^c);
    C = (1-lambdaL)*log(dm*lambdaL^e*NLV^f*NFR^gm);
   
    if HL_0 < lambdaL
        HL_0 = lambdaL;                                 % Boundary Condition
    end
    
    if C < 0
        C = 0;                                          % Boundary Condition
    end
    
    thetha = 1 + C*(sind(1.8*phi)-(sind(1.8*phi)^3)/3); % Correction factor, inclination
    HL = HL_0*thetha;                                   % Correction for inclined flow
	
end

% CAN BE IMPLEMENTED IN FUTURE, THREE PHASE FLOW

% g = 9.81;
% 
% qO = mO/rhoO;                               % Oil Volumetric Flow, m3/hr
% qW = mW/rhoW;                               % Water Volumetric Flow, m3/hr
% qG = mG/rhoG;                               % Gas Volumetric Flow, m3/hr
% 
% Vsg = qG/3600/(pi()/4*d^2);                  % Gas Superficial Velocity, m/s
% Vsl = (qO+qW)/3600/(pi()/4*d^2);             % Liquid Superficial Velocity, m/s
% 
% VW = mW/(mO+mW);                            % Water Weight Fraction
% VO = 1-VW;                                  % Oil Weight Fraction
% 
% rhoL = 1/((VW/rhoW)+(VO/rhoO));              % Liquid Mixture Density, kg/m3
% FrG = Vsg*sqrt(rhoG/(rhoL-rhoG)/9.81/d);     % Gas Froude No
% FrL = Vsl*sqrt(rhoL/(rhoL-rhoG)/9.81/d);     % Liquid Froude No
% 
% WW = mW/(mG + mW + mO);                     % Water Weight Fraction
% WO = mO/(mG + mW + mO);                     % Oil Weight Fraction
% WG = mG/(mG + mW + mO);                     % Gas Weight Fraction
% 
% rhom = 1/((WW/rhoW)+(WO/rhoO)+(WG/rhoG));   % Mixture Density, kg/m3
% Vm = (qO + qW + qG)/3600/(pi()/4*d^2);      % Mixture Velocity, m/sec
% 
% Frm = Vm^2/g/d;
% CL = Vsl/Vm;                                % Input Liquid Volume Fraction
