% Function to calculate fluid properties at a given pressure and
% temperature. Relations found in Beggs-Brill Chapter 1-16

% Input, fluid properties at standard condition:
% yG        -   Specific Gravity Gas
% yO        -   Specific Gravity Oil/Liquid
% qO_SC     -   Liquid Flow @ SC, stb/day
% qG_SC     -   Gas Flow @ SC, scf/day
% p         -   Pressure, psi
% T         -   Temperature, Farenheit
% Rs        -   Solution Gas-Oil Ratio, scf/bbl
% Bo        -   Formation Volume Factor, bbl/bbl
% z         -   Gas Compressibility Factor

% Output, fluid properties at P,T:
% qG/qO/qT          -   Gas/Liquid/Total Flow Rate, cuft/sec
% rhoG/rhoO/rhoM    -   Gas/Liquid/Mixture Density, lbm/cuft

function [qG, qO, qT, rhoO, rhoG, rhoM] = fluid_properties(yG, yO, qO_SC, qG_SC, p, T, Rs, Bo, z, GOR)

rhoO = (350*yO+0.0764*Rs*yG)/(5.615*Bo);                % Liquid Density @ Pavg, Tavg, lbm/cuft
rhoG = 2.7*p*yG/(z*(T+459.67));                         % Gas Density @ Pavg, Tavg, lbm/cuft
qO = 6.49*10^-5*qO_SC*Bo;                               % Liquid Flow @ Pavg, Tavg, ft3/sec

qG = 3.27*10^-7*z*(qG_SC-qO_SC*Rs)*(T+459.67)/p;        % Gas Flow @ Pavg, Tavg, ft3/sec

%%
% 
% $$e^{\pi i} + 1 = 0$$
% 

Bg = (14.7/p)*((T+459.67)/519.67)*z;            % cuft/scf
qG = qO_SC*(GOR-Rs)*Bg/(3600*24);               % cuft/sec

if qO < 0;                                              % Correction for negative flow
    qO = 0;
end
if qG < 0;                                              % Correction for negative flow
    qG = 0;
end

qT = qO + qG;                                           % Total Flow, ft3/sec
rhoM = (qO*rhoO+qG*rhoG)/qT;                            % Homogenous Mixture Density, lbm/cuft

end

