% Function which loads a matrix with information about the well depths and
% assigns it into a new matrix with additional calculated data

% Input:
% Tsurf     -   Surface Temperature, Farenheit
% TempGrad  -   Linear Temperature Gradient, Farenheit/100-ft

% Output:
% dsN       -   Matrix with the following information for each column:
    % Col1  -   Interval Number
    % Col2  -   Measured Depth In, ft
    % Col3  -   Measured Depth Out, ft
    % Col4  -   Length of Interval, ft
    % Col5  -   True Vertical Depth In, ft
    % Col6  -   True Vertical Depth Out, ft
    % Col7  -   Detpth Change, ft
    % Col8  -   Average Vertical Depth, ft
    % Col9  -   Average Temperature, F
    % Col10 -   Outer Diameter Annulus (Hole Size), ft
    % Col11 -   Inner Diameter Annulus (Drill Pipe OD), ft
    % Col12 -   Annular Capacity, sqft
    % Col13 -   Inclination, degrees
    % Col14 -   Guessed Pressure Change, psi
    % Col15 -   Inflow Pressure, psi
    % Col16 -   Outflow Pressure, psi
    % Col17 -   Average Pressure, psi
    % Col18 -   Pressure Gradient, psi
    % Col19 -   Pressure Change, psi
    % Col20 -   Iterations on interval

function dsN = newgrid_GUI(t)

load SIMULATIONTEST2
Welldata = WellPath;
intervals = size(Welldata, 1)-1;                    % Finds the numbers of depth intervals
dsN = nan(intervals,20);                            % Assigns dummy data to the output matrix
dsN(:,1) = transpose(1:intervals);                  % Numbers each interval

if t == 0                                           % Initial timestep simulated downwards
    dsN(:,2) = double(Welldata(1:intervals,2));     % Measured depth inflow, ft
    dsN(:,3) = double(Welldata(2:intervals+1,2));   % Measured depth outflow, ft
    dsN(:,5) = double(Welldata(1:intervals,5));     % TVD inflow, ft
    dsN(:,6) = double(Welldata(2:intervals+1,5));   % TVD outflow, ft
else                                                % All other timesteps, simultated upwards
    dsN(:,2) = double(Welldata(2:intervals+1,2));   % Measured depth inflow, ft
    dsN(:,3) = double(Welldata(1:intervals,2));     % Measured depth outflow, ft
    dsN(:,5) = double(Welldata(2:intervals+1,5));   % TVD inflow, ft
    dsN(:,6) = double(Welldata(1:intervals,5));     % TVD outflow, ft
end

for i = 1:intervals      
    MD = dsN(i,2);
    found = 0;
    index = 1;
    while found == 0;
        if Geometry(index,1)<(MD-0.001);
            index = index + 1;
        else
            found = 1;
        end
    end
    d1 = Geometry(index,2)/12;
    d2 = Geometry(index,3)/12;
    d3 = Geometry(index,4)/12;
    dsN(i,11) = d2; 
    dsN(i,10) = d3;
end

dsN(:,4) = abs(dsN(:,3) - dsN(:,2));                % Length of interval, ft
dsN(:,7) = dsN(:,6) - dsN(:,5);                     % Depth change interval, ft
dsN(:,13) = asind(dsN(:,7)./dsN(:,4));              % Calculted Inclination, degrees
dsN(:,12) = pi/4*(dsN(:,10).^2-dsN(:,11).^2);       % Flow Area Annulus, sqft  
dsN(:,8) = (dsN(:,5) + dsN(:,6))/2;                 % Average TVD, ft
%dsN(:,9) = Tsurf + dsN(:,8).*TempGrad;              % Average temperature, F
dsN(:,9) = interp1(TempTable(:,1),TempTable(:,3),dsN(:,8));
dsN(:,20) = zeros(intervals, 1);                    % Iteration counter defined
dsN(:,14) = zeros(intervals, 1);                    % First pressure change guessed, psi
dsN(:,19) = zeros(intervals,1);                     % Calculated pressure defined, psi


% header = {'Interval' 'MDin' 'MDout' 'dZ' 'TVDin' 'TVDout' 'dh' ...
%           'TVDavg' 'TavgF' 'dHS' 'doDS' 'A' 'phi' 'dp_guess'...
%           'pin' 'pout' 'pavg' 'dPdZ' 'dP' 'iter'};
% data = nan(intervals, length(header));
% dsN = dataset({data,header{:}});

end 


