% Function calculates oil viscosity for a given pressure and temperature

% Beggs and Robinson Model for Dead Oil Viscosity (below bubblepoint)
%       Reference: Whitson: p37
% Vazquez and Beggs Model for Live Oil Viscosity (above bubblepoint)
%       Reference: Whitson: p38

% Input:
% Rs        -   Solution GOR, scf/STB
% API       -   Stock tank oil gravity, API
% T         -   Temperature, F 
% p         -   Pressure, psi
% pb        -   Bubble Point Pressure, psi

% Output:
% visOil    -   Oil Viscosity, cp    

function visOil = oilviscosity(API,T,Rs,p,pb)

% Beggs and Robinson Model
    x = T^(-1.163)*exp(6.9824-0.04658*API);
    A = 10.715*(Rs+100)^(-0.515); 
    B = 5.44*(Rs+150)^(-0.338); 
    visOil = A*(10^x-1.0)^B;                            % Saturated oil viscosity, cp

% Vazquez and Beggs Model
    if p > pb
        m = 2.6*p^(1.187)*exp(-11.513-(8.98*10^-5)*p);  % Correction parameter
        visOil = visOil*(p/pb)^m;                       % Live oil viscosity, cp
    end

end

