% Function Calculates the Gas Compressibility Factor
% From Pseudo-Reduced Properties Using the DAK-EOS

% Input:
% T_pr      -   Pseudo-Reduced Temperature
% P_pr      -   Pseudo-Reduced Pressure

% Output:
% Z_calc    -   Gas Compressibility Factor

function Z_calc = zfac5(T_pr,P_pr)

% Section Below Assigns the Coefficents Used in DAK-EOS

A1 = 0.3265;
A2 = -1.07;
A3 = -0.5339;
A4 = 0.01569;
A5 = -0.05165;
A6 = 0.5475;
A7 = -0.7361;
A8 = 0.1844;
A9 = 0.1056;
A10 = 0.6134;
A11 = 0.721;

% Section Below Iteratates Until a Valid Solution is Found

limit = 0.0001; % Iteration Limit, Max Z Deviation 
dev = 100;      % Initial Deviation Value, Set Above Limit
Z_guess = 1;    % Initial Compressibility Factor Guess
iter = 0;       % Iteration Counter Initiated


while abs(dev) > limit

    iter = iter + 1;                        % Iteration Counter Increased
    rho_pr = 0.27*P_pr/(Z_guess*T_pr);      % Pseudo-Reduced Density
    
    % Section Below Calculates Subsections of the Compressibility Factor
    c1 = A1 + A2*T_pr^-1 + A3*T_pr^-3 ...
        + A4*T_pr^-4 + A5*T_pr^-5;
    c2 = A6 + A7/T_pr + A8/T_pr^2;
    c3 = A9*(A7/T_pr + A8/T_pr^2);
    c4 = A10*(1+A11*rho_pr^2)*(rho_pr^2*T_pr^-3)...
        *exp(-A11*rho_pr^2);
    
    Z_calc = 1 + c1*rho_pr + ...
        c2*rho_pr^2 - c3*rho_pr^5 + c4;    % Compressibility Factor Calculated
    dev = Z_calc - Z_guess;                % Deviation From Guessed Value
    
    if iter < 200                                               
        Z_guess = Z_guess + dev;           % New Compressibility Factor Guess is Made
    else
        Z_guess = Z_guess + 0.01*dev;      % If Iterations Exceede 200, Brute-Force is Used
    end
           
    
end

end

